#!/usr/local/bin/python3.3
#
"""Demonstrate the Sankey class by producing three basic diagrams.
"""
import numpy as np
import matplotlib.pyplot as plt
from pylab import *
from matplotlib.sankey import Sankey

TotalTotal = 612
TotalWithImplementation = 514
TotalWithNoImplementation = 98
TotalWithAvailableCode = 226
TotalWithNoAvailableCode = TotalWithImplementation - TotalWithAvailableCode
TotalBuildCompiles = 111
TotalBuildRuns = 92
TotalBuildOnlineRuns = 5
TotalBuildSuccess = 97
TotalBuildDoesNotCompile = 111
TotalBuildDoesNotRun = 92
TotalBuildAttempted = 223
TotalBuildDoesNotCompile = TotalBuildAttempted - TotalBuildCompiles 
TotalBuildCompilesDoesNotRun = TotalBuildCompiles - TotalBuildSuccess

TotalWithArticleCode = 84
TotalWithGoogleCode = 65
TotalWithEmailCode = 77

# Example 1 -- Mostly defaults
# This demonstrates how to create a simple diagram by implicitly calling the
# Sankey.add() method and by appending finish() to the call to the class.
fig = plt.figure()
ax = fig.add_subplot(1, 1, 1, xticks=[], yticks=[], title="")
sankey = Sankey(ax=ax, scale=0.8, offset=0.2, head_angle=90,
                format='%.0f', unit=None)
########################################################################
#sankey.add(
#   flows=[
#      TotalTotal/TotalTotal,
#         -TotalWithNoImplementation/TotalTotal,
#            -TotalWithNoAvailableCode/TotalTotal,
#               -TotalBuildDoesNotCompile/TotalTotal,
#               -TotalBuildCompilesDoesNotRun/TotalTotal,
#               -TotalBuildRuns/TotalTotal,
#               -TotalBuildOnlineRuns/TotalTotal
#    ],
#    labels=[
#       str(TotalTotal), 
#       'Theoretical\n' + str(TotalWithNoImplementation), 
#       'No Code\nAvailable\n' + str(TotalWithNoAvailableCode), 
#       'Build\nfails\n' + str(TotalBuildDoesNotCompile), 
#       'Runtime\nfails\n' + str(TotalBuildCompilesDoesNotRun), 
#       'Builds\n& runs\n' + str(TotalBuildRuns), 
#       'Online\nruns\n' + str(TotalBuildOnlineRuns)
#    ],
#    orientations=[
#         0, 1, 0, -1, -1, 1, 1
#    ],
#   pathlengths = [0.0, 0.1, 0.1, 0.2, 0.2, 0.2, 0.2]
#)
########################################################################
# diagram 0
########################################################################
sankey.add(
   flows=[
      TotalTotal/TotalTotal,
         -TotalWithNoImplementation/TotalTotal,
         -TotalWithImplementation/TotalTotal,
    ],
    labels=[
       str(TotalTotal), 
       'Theoretical\n' + str(TotalWithNoImplementation), 
        '', 
    ],
    orientations=[
         0, 1, 0
    ],
#   pathlengths = [0.0, 0.1, 0.2, 0.2]
    linewidth=0.0,
    trunklength=1.3
)
########################################################################
# diagram 1
########################################################################
sankey.add(
   flows=[
       TotalWithImplementation/TotalTotal,
          -TotalWithArticleCode/TotalTotal,
          -TotalWithGoogleCode/TotalTotal,
          -TotalWithEmailCode/TotalTotal,
          -TotalWithNoAvailableCode/TotalTotal,
    ],
    labels=[
       str(TotalWithImplementation), 
       'Article\n' + str(TotalWithArticleCode), 
       'Web\n' + str(TotalWithGoogleCode), 
       'Email\n' + str(TotalWithEmailCode), 
       'Code\nUnavailable\n' + str(TotalWithNoAvailableCode), 
    ],
    orientations=[
         0, 1, 1, 1, -1
    ],
   prior=0, 
   connect=(2, 0),
   trunklength = 1.0,
   linewidth=0.0,
#   pathlengths = [0.0, 0.1, 0.1, 0.1]
)
########################################################################
# diagram 2
########################################################################
sankey.add(
   flows=[
          TotalWithArticleCode/TotalTotal,
          TotalWithGoogleCode/TotalTotal,
          TotalWithEmailCode/TotalTotal,
          -TotalWithAvailableCode/TotalTotal
    ],
    labels=[
       '',
       '',
       '',
       str(TotalWithAvailableCode)
    ],
    orientations=[
         -1, -1, -1, 0
    ],
   prior=1, 
   connect=(1, 0),
   trunklength = 0.6,
   linewidth=0.0,
#   pathlengths = [0.4, 0.4, 0.4, 0.0]
)
 
########################################################################
# diagram 3
########################################################################
sankey.add(
   flows=[
       TotalWithAvailableCode/TotalTotal,
           -TotalBuildDoesNotCompile/TotalTotal,
           -TotalBuildCompilesDoesNotRun/TotalTotal,
           -TotalBuildRuns/TotalTotal,
           -TotalBuildOnlineRuns/TotalTotal
    ],
    labels=[
       '',
       'Build\nfails\n' + str(TotalBuildDoesNotCompile), 
       'Run\nfails\n' + str(TotalBuildCompilesDoesNotRun), 
       'Builds\n& runs\n' + str(TotalBuildRuns), 
       'Online\nruns\n' + str(TotalBuildOnlineRuns)
    ],
    orientations=[
        0, -1, -1, 1, 1
    ],
   prior=2, 
   connect=(3, 0),
   trunklength = 0.9,
   linewidth=0.0,
   pathlengths = [0.0, 0.4, 0.2, 0.4, 0.2]
)


########################################################################
diagrams = sankey.finish()

diagrams[0].patch.set_facecolor('#80b060')
diagrams[1].patch.set_facecolor('#90c070')
diagrams[2].patch.set_facecolor('#90c070')
diagrams[3].patch.set_facecolor('#a0d080') 

diagrams[1].texts[4].set_color('r')
diagrams[3].texts[1].set_color('r')
diagrams[3].texts[2].set_color('r')

fig.patch.set_visible(False)
ax.axis('off')
#plt.savefig(filename='fullflow.pdf',frameon=False) # 
plt.show()
