#!/usr/local/bin/python3.3

import numpy as np
import matplotlib.pyplot as plt
from pylab import *
from matplotlib.sankey import Sankey
import sys

##################################################################
# Command line parsing
what = sys.argv[1]  # Total, NSF, Pldi, ...
kind = sys.argv[2]  # small or large

if kind == 'large':
   titleOn = True
   SCALE = 0.8
else:
   titleOn = False
   SCALE = 0.4

OutputFile = 'sankey-' + what + '.pdf'

##################################################################

##################################################################
# This code has been inserted automatically.
data = {}
data['TissecTotal'] = 13
data['TissecWithImplementation'] = 6
data['TissecWithImplementationPercentOfTotal'] = 46.2
data['TissecWithNoImplementation'] = 7
data['TissecWithNoImplementationPercentOfTotal'] = 53.8
data['TissecWithAvailableCode'] = 4
data['TissecWithAvailableCodePercentOfImplementation'] = 66.7
data['TissecWithNoAvailableCode'] = 2
data['TissecWithNoAvailableCodePercentOfImplementation'] = 33.3
data['TissecWithArticleCode'] = 3
data['TissecWithArticleCodePercentOfImplementation'] = 50.0
data['TissecWithArticleCodePercentOfAvailableCode'] = 75.0
data['TissecWithGoogleCode'] = 0
data['TissecWithGoogleCodePercentOfImplementation'] = 0.0
data['TissecWithGoogleCodePercentOfAvailableCode'] = 0.0
data['TissecWithEmailCode'] = 1
data['TissecWithEmailCodePercentOfImplementation'] = 16.7
data['TissecWithEmailCodePercentOfAvailableCode'] = 25.0
data['TacoTotal'] = 60
data['TacoWithImplementation'] = 39
data['TacoWithImplementationPercentOfTotal'] = 65.0
data['TacoWithNoImplementation'] = 21
data['TacoWithNoImplementationPercentOfTotal'] = 35.0
data['TacoWithAvailableCode'] = 14
data['TacoWithAvailableCodePercentOfImplementation'] = 35.9
data['TacoWithNoAvailableCode'] = 25
data['TacoWithNoAvailableCodePercentOfImplementation'] = 64.1
data['TacoWithArticleCode'] = 7
data['TacoWithArticleCodePercentOfImplementation'] = 17.9
data['TacoWithArticleCodePercentOfAvailableCode'] = 50.0
data['TacoWithGoogleCode'] = 3
data['TacoWithGoogleCodePercentOfImplementation'] = 7.7
data['TacoWithGoogleCodePercentOfAvailableCode'] = 21.4
data['TacoWithEmailCode'] = 4
data['TacoWithEmailCodePercentOfImplementation'] = 10.3
data['TacoWithEmailCodePercentOfAvailableCode'] = 28.6
data['TodsTotal'] = 29
data['TodsWithImplementation'] = 17
data['TodsWithImplementationPercentOfTotal'] = 58.6
data['TodsWithNoImplementation'] = 12
data['TodsWithNoImplementationPercentOfTotal'] = 41.4
data['TodsWithAvailableCode'] = 8
data['TodsWithAvailableCodePercentOfImplementation'] = 47.1
data['TodsWithNoAvailableCode'] = 9
data['TodsWithNoAvailableCodePercentOfImplementation'] = 52.9
data['TodsWithArticleCode'] = 1
data['TodsWithArticleCodePercentOfImplementation'] = 5.9
data['TodsWithArticleCodePercentOfAvailableCode'] = 12.5
data['TodsWithGoogleCode'] = 5
data['TodsWithGoogleCodePercentOfImplementation'] = 29.4
data['TodsWithGoogleCodePercentOfAvailableCode'] = 62.5
data['TodsWithEmailCode'] = 2
data['TodsWithEmailCodePercentOfImplementation'] = 11.8
data['TodsWithEmailCodePercentOfAvailableCode'] = 25.0
data['OsdiTotal'] = 24
data['OsdiWithImplementation'] = 24
data['OsdiWithImplementationPercentOfTotal'] = 100.0
data['OsdiWithNoImplementation'] = 0
data['OsdiWithNoImplementationPercentOfTotal'] = 0.0
data['OsdiWithAvailableCode'] = 10
data['OsdiWithAvailableCodePercentOfImplementation'] = 41.7
data['OsdiWithNoAvailableCode'] = 14
data['OsdiWithNoAvailableCodePercentOfImplementation'] = 58.3
data['OsdiWithArticleCode'] = 4
data['OsdiWithArticleCodePercentOfImplementation'] = 16.7
data['OsdiWithArticleCodePercentOfAvailableCode'] = 40.0
data['OsdiWithGoogleCode'] = 4
data['OsdiWithGoogleCodePercentOfImplementation'] = 16.7
data['OsdiWithGoogleCodePercentOfAvailableCode'] = 40.0
data['OsdiWithEmailCode'] = 2
data['OsdiWithEmailCodePercentOfImplementation'] = 8.3
data['OsdiWithEmailCodePercentOfAvailableCode'] = 20.0
data['SigmodTotal'] = 46
data['SigmodWithImplementation'] = 45
data['SigmodWithImplementationPercentOfTotal'] = 97.8
data['SigmodWithNoImplementation'] = 1
data['SigmodWithNoImplementationPercentOfTotal'] = 2.2
data['SigmodWithAvailableCode'] = 17
data['SigmodWithAvailableCodePercentOfImplementation'] = 37.8
data['SigmodWithNoAvailableCode'] = 28
data['SigmodWithNoAvailableCodePercentOfImplementation'] = 62.2
data['SigmodWithArticleCode'] = 3
data['SigmodWithArticleCodePercentOfImplementation'] = 6.7
data['SigmodWithArticleCodePercentOfAvailableCode'] = 17.6
data['SigmodWithGoogleCode'] = 6
data['SigmodWithGoogleCodePercentOfImplementation'] = 13.3
data['SigmodWithGoogleCodePercentOfAvailableCode'] = 35.3
data['SigmodWithEmailCode'] = 8
data['SigmodWithEmailCodePercentOfImplementation'] = 17.8
data['SigmodWithEmailCodePercentOfAvailableCode'] = 47.1
data['AsplosTotal'] = 37
data['AsplosWithImplementation'] = 30
data['AsplosWithImplementationPercentOfTotal'] = 81.1
data['AsplosWithNoImplementation'] = 7
data['AsplosWithNoImplementationPercentOfTotal'] = 18.9
data['AsplosWithAvailableCode'] = 7
data['AsplosWithAvailableCodePercentOfImplementation'] = 23.3
data['AsplosWithNoAvailableCode'] = 23
data['AsplosWithNoAvailableCodePercentOfImplementation'] = 76.7
data['AsplosWithArticleCode'] = 2
data['AsplosWithArticleCodePercentOfImplementation'] = 6.7
data['AsplosWithArticleCodePercentOfAvailableCode'] = 28.6
data['AsplosWithGoogleCode'] = 1
data['AsplosWithGoogleCodePercentOfImplementation'] = 3.3
data['AsplosWithGoogleCodePercentOfAvailableCode'] = 14.3
data['AsplosWithEmailCode'] = 4
data['AsplosWithEmailCodePercentOfImplementation'] = 13.3
data['AsplosWithEmailCodePercentOfAvailableCode'] = 57.1
data['OopslaTotal'] = 81
data['OopslaWithImplementation'] = 66
data['OopslaWithImplementationPercentOfTotal'] = 81.5
data['OopslaWithNoImplementation'] = 15
data['OopslaWithNoImplementationPercentOfTotal'] = 18.5
data['OopslaWithAvailableCode'] = 45
data['OopslaWithAvailableCodePercentOfImplementation'] = 68.2
data['OopslaWithNoAvailableCode'] = 21
data['OopslaWithNoAvailableCodePercentOfImplementation'] = 31.8
data['OopslaWithArticleCode'] = 30
data['OopslaWithArticleCodePercentOfImplementation'] = 45.5
data['OopslaWithArticleCodePercentOfAvailableCode'] = 66.7
data['OopslaWithGoogleCode'] = 6
data['OopslaWithGoogleCodePercentOfImplementation'] = 9.1
data['OopslaWithGoogleCodePercentOfAvailableCode'] = 13.3
data['OopslaWithEmailCode'] = 9
data['OopslaWithEmailCodePercentOfImplementation'] = 13.6
data['OopslaWithEmailCodePercentOfAvailableCode'] = 20.0
data['ToplasTotal'] = 15
data['ToplasWithImplementation'] = 10
data['ToplasWithImplementationPercentOfTotal'] = 66.7
data['ToplasWithNoImplementation'] = 5
data['ToplasWithNoImplementationPercentOfTotal'] = 33.3
data['ToplasWithAvailableCode'] = 8
data['ToplasWithAvailableCodePercentOfImplementation'] = 80.0
data['ToplasWithNoAvailableCode'] = 2
data['ToplasWithNoAvailableCodePercentOfImplementation'] = 20.0
data['ToplasWithArticleCode'] = 6
data['ToplasWithArticleCodePercentOfImplementation'] = 60.0
data['ToplasWithArticleCodePercentOfAvailableCode'] = 75.0
data['ToplasWithGoogleCode'] = 1
data['ToplasWithGoogleCodePercentOfImplementation'] = 10.0
data['ToplasWithGoogleCodePercentOfAvailableCode'] = 12.5
data['ToplasWithEmailCode'] = 1
data['ToplasWithEmailCodePercentOfImplementation'] = 10.0
data['ToplasWithEmailCodePercentOfAvailableCode'] = 12.5
data['PldiTotal'] = 48
data['PldiWithImplementation'] = 48
data['PldiWithImplementationPercentOfTotal'] = 100.0
data['PldiWithNoImplementation'] = 0
data['PldiWithNoImplementationPercentOfTotal'] = 0.0
data['PldiWithAvailableCode'] = 26
data['PldiWithAvailableCodePercentOfImplementation'] = 54.2
data['PldiWithNoAvailableCode'] = 22
data['PldiWithNoAvailableCodePercentOfImplementation'] = 45.8
data['PldiWithArticleCode'] = 10
data['PldiWithArticleCodePercentOfImplementation'] = 20.8
data['PldiWithArticleCodePercentOfAvailableCode'] = 38.5
data['PldiWithGoogleCode'] = 6
data['PldiWithGoogleCodePercentOfImplementation'] = 12.5
data['PldiWithGoogleCodePercentOfAvailableCode'] = 23.1
data['PldiWithEmailCode'] = 10
data['PldiWithEmailCodePercentOfImplementation'] = 20.8
data['PldiWithEmailCodePercentOfAvailableCode'] = 38.5
data['SospTotal'] = 28
data['SospWithImplementation'] = 26
data['SospWithImplementationPercentOfTotal'] = 92.9
data['SospWithNoImplementation'] = 2
data['SospWithNoImplementationPercentOfTotal'] = 7.1
data['SospWithAvailableCode'] = 9
data['SospWithAvailableCodePercentOfImplementation'] = 34.6
data['SospWithNoAvailableCode'] = 17
data['SospWithNoAvailableCodePercentOfImplementation'] = 65.4
data['SospWithArticleCode'] = 2
data['SospWithArticleCodePercentOfImplementation'] = 7.7
data['SospWithArticleCodePercentOfAvailableCode'] = 22.2
data['SospWithGoogleCode'] = 4
data['SospWithGoogleCodePercentOfImplementation'] = 15.4
data['SospWithGoogleCodePercentOfAvailableCode'] = 44.4
data['SospWithEmailCode'] = 3
data['SospWithEmailCodePercentOfImplementation'] = 11.5
data['SospWithEmailCodePercentOfAvailableCode'] = 33.3
data['VldbTotal'] = 141
data['VldbWithImplementation'] = 134
data['VldbWithImplementationPercentOfTotal'] = 95.0
data['VldbWithNoImplementation'] = 7
data['VldbWithNoImplementationPercentOfTotal'] = 5.0
data['VldbWithAvailableCode'] = 50
data['VldbWithAvailableCodePercentOfImplementation'] = 37.3
data['VldbWithNoAvailableCode'] = 84
data['VldbWithNoAvailableCodePercentOfImplementation'] = 62.7
data['VldbWithArticleCode'] = 9
data['VldbWithArticleCodePercentOfImplementation'] = 6.7
data['VldbWithArticleCodePercentOfAvailableCode'] = 18.0
data['VldbWithGoogleCode'] = 21
data['VldbWithGoogleCodePercentOfImplementation'] = 15.7
data['VldbWithGoogleCodePercentOfAvailableCode'] = 42.0
data['VldbWithEmailCode'] = 20
data['VldbWithEmailCodePercentOfImplementation'] = 14.9
data['VldbWithEmailCodePercentOfAvailableCode'] = 40.0
data['TocsTotal'] = 14
data['TocsWithImplementation'] = 13
data['TocsWithImplementationPercentOfTotal'] = 92.9
data['TocsWithNoImplementation'] = 1
data['TocsWithNoImplementationPercentOfTotal'] = 7.1
data['TocsWithAvailableCode'] = 6
data['TocsWithAvailableCodePercentOfImplementation'] = 46.2
data['TocsWithNoAvailableCode'] = 7
data['TocsWithNoAvailableCodePercentOfImplementation'] = 53.8
data['TocsWithArticleCode'] = 3
data['TocsWithArticleCodePercentOfImplementation'] = 23.1
data['TocsWithArticleCodePercentOfAvailableCode'] = 50.0
data['TocsWithGoogleCode'] = 2
data['TocsWithGoogleCodePercentOfImplementation'] = 15.4
data['TocsWithGoogleCodePercentOfAvailableCode'] = 33.3
data['TocsWithEmailCode'] = 1
data['TocsWithEmailCodePercentOfImplementation'] = 7.7
data['TocsWithEmailCodePercentOfAvailableCode'] = 16.7
data['CcsTotal'] = 76
data['CcsWithImplementation'] = 56
data['CcsWithImplementationPercentOfTotal'] = 73.7
data['CcsWithNoImplementation'] = 20
data['CcsWithNoImplementationPercentOfTotal'] = 26.3
data['CcsWithAvailableCode'] = 24
data['CcsWithAvailableCodePercentOfImplementation'] = 42.9
data['CcsWithNoAvailableCode'] = 32
data['CcsWithNoAvailableCodePercentOfImplementation'] = 57.1
data['CcsWithArticleCode'] = 4
data['CcsWithArticleCodePercentOfImplementation'] = 7.1
data['CcsWithArticleCodePercentOfAvailableCode'] = 16.7
data['CcsWithGoogleCode'] = 6
data['CcsWithGoogleCodePercentOfImplementation'] = 10.7
data['CcsWithGoogleCodePercentOfAvailableCode'] = 25.0
data['CcsWithEmailCode'] = 14
data['CcsWithEmailCodePercentOfImplementation'] = 25.0
data['CcsWithEmailCodePercentOfAvailableCode'] = 58.3
data['JournalsTotal'] = 131
data['JournalsWithImplementation'] = 85
data['JournalsWithImplementationPercentOfTotal'] = 64.9
data['JournalsWithNoImplementation'] = 46
data['JournalsWithNoImplementationPercentOfTotal'] = 35.1
data['JournalsWithAvailableCode'] = 40
data['JournalsWithAvailableCodePercentOfImplementation'] = 47.1
data['JournalsWithNoAvailableCode'] = 45
data['JournalsWithNoAvailableCodePercentOfImplementation'] = 52.9
data['JournalsWithArticleCode'] = 20
data['JournalsWithArticleCodePercentOfImplementation'] = 23.5
data['JournalsWithArticleCodePercentOfAvailableCode'] = 50.0
data['JournalsWithGoogleCode'] = 11
data['JournalsWithGoogleCodePercentOfImplementation'] = 12.9
data['JournalsWithGoogleCodePercentOfAvailableCode'] = 27.5
data['JournalsWithEmailCode'] = 9
data['JournalsWithEmailCodePercentOfImplementation'] = 10.6
data['JournalsWithEmailCodePercentOfAvailableCode'] = 22.5
data['ConferencesTotal'] = 481
data['ConferencesWithImplementation'] = 429
data['ConferencesWithImplementationPercentOfTotal'] = 89.2
data['ConferencesWithNoImplementation'] = 52
data['ConferencesWithNoImplementationPercentOfTotal'] = 10.8
data['ConferencesWithAvailableCode'] = 188
data['ConferencesWithAvailableCodePercentOfImplementation'] = 43.8
data['ConferencesWithNoAvailableCode'] = 241
data['ConferencesWithNoAvailableCodePercentOfImplementation'] = 56.2
data['ConferencesWithArticleCode'] = 64
data['ConferencesWithArticleCodePercentOfImplementation'] = 14.9
data['ConferencesWithArticleCodePercentOfAvailableCode'] = 34.0
data['ConferencesWithGoogleCode'] = 54
data['ConferencesWithGoogleCodePercentOfImplementation'] = 12.6
data['ConferencesWithGoogleCodePercentOfAvailableCode'] = 28.7
data['ConferencesWithEmailCode'] = 70
data['ConferencesWithEmailCodePercentOfImplementation'] = 16.3
data['ConferencesWithEmailCodePercentOfAvailableCode'] = 37.2
data['CommercialNoneTotal'] = 414
data['CommercialNoneWithImplementation'] = 349
data['CommercialNoneWithImplementationPercentOfTotal'] = 84.3
data['CommercialNoneWithNoImplementation'] = 65
data['CommercialNoneWithNoImplementationPercentOfTotal'] = 15.7
data['CommercialNoneWithAvailableCode'] = 178
data['CommercialNoneWithAvailableCodePercentOfImplementation'] = 51.0
data['CommercialNoneWithNoAvailableCode'] = 171
data['CommercialNoneWithNoAvailableCodePercentOfImplementation'] = 49.0
data['CommercialNoneWithArticleCode'] = 65
data['CommercialNoneWithArticleCodePercentOfImplementation'] = 18.6
data['CommercialNoneWithArticleCodePercentOfAvailableCode'] = 36.5
data['CommercialNoneWithGoogleCode'] = 53
data['CommercialNoneWithGoogleCodePercentOfImplementation'] = 15.2
data['CommercialNoneWithGoogleCodePercentOfAvailableCode'] = 29.8
data['CommercialNoneWithEmailCode'] = 60
data['CommercialNoneWithEmailCodePercentOfImplementation'] = 17.2
data['CommercialNoneWithEmailCodePercentOfAvailableCode'] = 33.7
data['CommercialPartTotal'] = 149
data['CommercialPartWithImplementation'] = 130
data['CommercialPartWithImplementationPercentOfTotal'] = 87.2
data['CommercialPartWithNoImplementation'] = 19
data['CommercialPartWithNoImplementationPercentOfTotal'] = 12.8
data['CommercialPartWithAvailableCode'] = 42
data['CommercialPartWithAvailableCodePercentOfImplementation'] = 32.3
data['CommercialPartWithNoAvailableCode'] = 88
data['CommercialPartWithNoAvailableCodePercentOfImplementation'] = 67.7
data['CommercialPartWithArticleCode'] = 13
data['CommercialPartWithArticleCodePercentOfImplementation'] = 10.0
data['CommercialPartWithArticleCodePercentOfAvailableCode'] = 31.0
data['CommercialPartWithGoogleCode'] = 12
data['CommercialPartWithGoogleCodePercentOfImplementation'] = 9.2
data['CommercialPartWithGoogleCodePercentOfAvailableCode'] = 28.6
data['CommercialPartWithEmailCode'] = 17
data['CommercialPartWithEmailCodePercentOfImplementation'] = 13.1
data['CommercialPartWithEmailCodePercentOfAvailableCode'] = 40.5
data['CommercialFullTotal'] = 49
data['CommercialFullWithImplementation'] = 35
data['CommercialFullWithImplementationPercentOfTotal'] = 71.4
data['CommercialFullWithNoImplementation'] = 14
data['CommercialFullWithNoImplementationPercentOfTotal'] = 28.6
data['CommercialFullWithAvailableCode'] = 8
data['CommercialFullWithAvailableCodePercentOfImplementation'] = 22.9
data['CommercialFullWithNoAvailableCode'] = 27
data['CommercialFullWithNoAvailableCodePercentOfImplementation'] = 77.1
data['CommercialFullWithArticleCode'] = 6
data['CommercialFullWithArticleCodePercentOfImplementation'] = 17.1
data['CommercialFullWithArticleCodePercentOfAvailableCode'] = 75.0
data['CommercialFullWithGoogleCode'] = 0
data['CommercialFullWithGoogleCodePercentOfImplementation'] = 0.0
data['CommercialFullWithGoogleCodePercentOfAvailableCode'] = 0.0
data['CommercialFullWithEmailCode'] = 2
data['CommercialFullWithEmailCodePercentOfImplementation'] = 5.7
data['CommercialFullWithEmailCodePercentOfAvailableCode'] = 25.0
data['NoNSFTotal'] = 357
data['NoNSFWithImplementation'] = 287
data['NoNSFWithImplementationPercentOfTotal'] = 80.4
data['NoNSFWithNoImplementation'] = 70
data['NoNSFWithNoImplementationPercentOfTotal'] = 19.6
data['NoNSFWithAvailableCode'] = 125
data['NoNSFWithAvailableCodePercentOfImplementation'] = 43.6
data['NoNSFWithNoAvailableCode'] = 162
data['NoNSFWithNoAvailableCodePercentOfImplementation'] = 56.4
data['NoNSFWithArticleCode'] = 48
data['NoNSFWithArticleCodePercentOfImplementation'] = 16.7
data['NoNSFWithArticleCodePercentOfAvailableCode'] = 38.4
data['NoNSFWithGoogleCode'] = 35
data['NoNSFWithGoogleCodePercentOfImplementation'] = 12.2
data['NoNSFWithGoogleCodePercentOfAvailableCode'] = 28.0
data['NoNSFWithEmailCode'] = 42
data['NoNSFWithEmailCodePercentOfImplementation'] = 14.6
data['NoNSFWithEmailCodePercentOfAvailableCode'] = 33.6
data['NSFTotal'] = 255
data['NSFWithImplementation'] = 227
data['NSFWithImplementationPercentOfTotal'] = 89.0
data['NSFWithNoImplementation'] = 28
data['NSFWithNoImplementationPercentOfTotal'] = 11.0
data['NSFWithAvailableCode'] = 103
data['NSFWithAvailableCodePercentOfImplementation'] = 45.4
data['NSFWithNoAvailableCode'] = 124
data['NSFWithNoAvailableCodePercentOfImplementation'] = 54.6
data['NSFWithArticleCode'] = 36
data['NSFWithArticleCodePercentOfImplementation'] = 15.9
data['NSFWithArticleCodePercentOfAvailableCode'] = 35.0
data['NSFWithGoogleCode'] = 30
data['NSFWithGoogleCodePercentOfImplementation'] = 13.2
data['NSFWithGoogleCodePercentOfAvailableCode'] = 29.1
data['NSFWithEmailCode'] = 37
data['NSFWithEmailCodePercentOfImplementation'] = 16.3
data['NSFWithEmailCodePercentOfAvailableCode'] = 35.9
data['TotalTotal'] = 612
data['TotalWithImplementation'] = 514
data['TotalWithImplementationPercentOfTotal'] = 84.0
data['TotalWithNoImplementation'] = 98
data['TotalWithNoImplementationPercentOfTotal'] = 16.0
data['TotalWithAvailableCode'] = 228
data['TotalWithAvailableCodePercentOfImplementation'] = 44.4
data['TotalWithNoAvailableCode'] = 286
data['TotalWithNoAvailableCodePercentOfImplementation'] = 55.6
data['TotalWithArticleCode'] = 84
data['TotalWithArticleCodePercentOfImplementation'] = 16.3
data['TotalWithArticleCodePercentOfAvailableCode'] = 36.8
data['TotalWithGoogleCode'] = 65
data['TotalWithGoogleCodePercentOfImplementation'] = 12.6
data['TotalWithGoogleCodePercentOfAvailableCode'] = 28.5
data['TotalWithEmailCode'] = 79
data['TotalWithEmailCodePercentOfImplementation'] = 15.4
data['TotalWithEmailCodePercentOfAvailableCode'] = 34.6
data['TissecBuildAttempted'] = 4
data['TissecBuildAttemptedPercentOfImplementation'] = 66.7
data['TissecBuildAttemptedPercentOfAvailableCode'] = 100.0
data['TissecBuildCompiles'] = 2
data['TissecBuildCompilesPercentOfAttempted'] = 50.0
data['TissecBuildDoesNotCompile'] = 2
data['TissecBuildDoesNotCompilePercentOfAttempted'] = 50.0
data['TissecBuildRuns'] = 2
data['TissecBuildRunsPercentOfAttempted'] = 50.0
data['TissecBuildCompilesDoesNotRun'] = 0
data['TissecBuildCompilesDoesNotRunPercentOfAttempted'] = 0.0
data['TissecBuildOnlineRuns'] = 0
data['TissecBuildOnlineRunsPercentOfAttempted'] = 0.0
data['TissecBuildSuccess'] = 2
data['TissecBuildSuccessPercentOfAttempted'] = 50.0
data['TissecBuildSuccessPercentOfImplementation'] = 33.3
data['TacoBuildAttempted'] = 14
data['TacoBuildAttemptedPercentOfImplementation'] = 35.9
data['TacoBuildAttemptedPercentOfAvailableCode'] = 100.0
data['TacoBuildCompiles'] = 6
data['TacoBuildCompilesPercentOfAttempted'] = 42.9
data['TacoBuildDoesNotCompile'] = 7
data['TacoBuildDoesNotCompilePercentOfAttempted'] = 50.0
data['TacoBuildRuns'] = 6
data['TacoBuildRunsPercentOfAttempted'] = 42.9
data['TacoBuildCompilesDoesNotRun'] = 0
data['TacoBuildCompilesDoesNotRunPercentOfAttempted'] = 0.0
data['TacoBuildOnlineRuns'] = 0
data['TacoBuildOnlineRunsPercentOfAttempted'] = 0.0
data['TacoBuildSuccess'] = 6
data['TacoBuildSuccessPercentOfAttempted'] = 42.9
data['TacoBuildSuccessPercentOfImplementation'] = 15.4
data['TodsBuildAttempted'] = 8
data['TodsBuildAttemptedPercentOfImplementation'] = 47.1
data['TodsBuildAttemptedPercentOfAvailableCode'] = 100.0
data['TodsBuildCompiles'] = 5
data['TodsBuildCompilesPercentOfAttempted'] = 62.5
data['TodsBuildDoesNotCompile'] = 3
data['TodsBuildDoesNotCompilePercentOfAttempted'] = 37.5
data['TodsBuildRuns'] = 5
data['TodsBuildRunsPercentOfAttempted'] = 62.5
data['TodsBuildCompilesDoesNotRun'] = 0
data['TodsBuildCompilesDoesNotRunPercentOfAttempted'] = 0.0
data['TodsBuildOnlineRuns'] = 0
data['TodsBuildOnlineRunsPercentOfAttempted'] = 0.0
data['TodsBuildSuccess'] = 5
data['TodsBuildSuccessPercentOfAttempted'] = 62.5
data['TodsBuildSuccessPercentOfImplementation'] = 29.4
data['OsdiBuildAttempted'] = 10
data['OsdiBuildAttemptedPercentOfImplementation'] = 41.7
data['OsdiBuildAttemptedPercentOfAvailableCode'] = 100.0
data['OsdiBuildCompiles'] = 6
data['OsdiBuildCompilesPercentOfAttempted'] = 60.0
data['OsdiBuildDoesNotCompile'] = 4
data['OsdiBuildDoesNotCompilePercentOfAttempted'] = 40.0
data['OsdiBuildRuns'] = 5
data['OsdiBuildRunsPercentOfAttempted'] = 50.0
data['OsdiBuildCompilesDoesNotRun'] = 1
data['OsdiBuildCompilesDoesNotRunPercentOfAttempted'] = 10.0
data['OsdiBuildOnlineRuns'] = 0
data['OsdiBuildOnlineRunsPercentOfAttempted'] = 0.0
data['OsdiBuildSuccess'] = 5
data['OsdiBuildSuccessPercentOfAttempted'] = 50.0
data['OsdiBuildSuccessPercentOfImplementation'] = 20.8
data['SigmodBuildAttempted'] = 17
data['SigmodBuildAttemptedPercentOfImplementation'] = 37.8
data['SigmodBuildAttemptedPercentOfAvailableCode'] = 100.0
data['SigmodBuildCompiles'] = 11
data['SigmodBuildCompilesPercentOfAttempted'] = 64.7
data['SigmodBuildDoesNotCompile'] = 6
data['SigmodBuildDoesNotCompilePercentOfAttempted'] = 35.3
data['SigmodBuildRuns'] = 9
data['SigmodBuildRunsPercentOfAttempted'] = 52.9
data['SigmodBuildCompilesDoesNotRun'] = 2
data['SigmodBuildCompilesDoesNotRunPercentOfAttempted'] = 11.8
data['SigmodBuildOnlineRuns'] = 0
data['SigmodBuildOnlineRunsPercentOfAttempted'] = 0.0
data['SigmodBuildSuccess'] = 9
data['SigmodBuildSuccessPercentOfAttempted'] = 52.9
data['SigmodBuildSuccessPercentOfImplementation'] = 20.0
data['AsplosBuildAttempted'] = 7
data['AsplosBuildAttemptedPercentOfImplementation'] = 23.3
data['AsplosBuildAttemptedPercentOfAvailableCode'] = 100.0
data['AsplosBuildCompiles'] = 4
data['AsplosBuildCompilesPercentOfAttempted'] = 57.1
data['AsplosBuildDoesNotCompile'] = 3
data['AsplosBuildDoesNotCompilePercentOfAttempted'] = 42.9
data['AsplosBuildRuns'] = 4
data['AsplosBuildRunsPercentOfAttempted'] = 57.1
data['AsplosBuildCompilesDoesNotRun'] = 0
data['AsplosBuildCompilesDoesNotRunPercentOfAttempted'] = 0.0
data['AsplosBuildOnlineRuns'] = 0
data['AsplosBuildOnlineRunsPercentOfAttempted'] = 0.0
data['AsplosBuildSuccess'] = 4
data['AsplosBuildSuccessPercentOfAttempted'] = 57.1
data['AsplosBuildSuccessPercentOfImplementation'] = 13.3
data['OopslaBuildAttempted'] = 45
data['OopslaBuildAttemptedPercentOfImplementation'] = 68.2
data['OopslaBuildAttemptedPercentOfAvailableCode'] = 100.0
data['OopslaBuildCompiles'] = 20
data['OopslaBuildCompilesPercentOfAttempted'] = 44.4
data['OopslaBuildDoesNotCompile'] = 22
data['OopslaBuildDoesNotCompilePercentOfAttempted'] = 48.9
data['OopslaBuildRuns'] = 18
data['OopslaBuildRunsPercentOfAttempted'] = 40.0
data['OopslaBuildCompilesDoesNotRun'] = 2
data['OopslaBuildCompilesDoesNotRunPercentOfAttempted'] = 4.4
data['OopslaBuildOnlineRuns'] = 3
data['OopslaBuildOnlineRunsPercentOfAttempted'] = 6.7
data['OopslaBuildSuccess'] = 21
data['OopslaBuildSuccessPercentOfAttempted'] = 46.7
data['OopslaBuildSuccessPercentOfImplementation'] = 31.8
data['ToplasBuildAttempted'] = 8
data['ToplasBuildAttemptedPercentOfImplementation'] = 80.0
data['ToplasBuildAttemptedPercentOfAvailableCode'] = 100.0
data['ToplasBuildCompiles'] = 3
data['ToplasBuildCompilesPercentOfAttempted'] = 37.5
data['ToplasBuildDoesNotCompile'] = 5
data['ToplasBuildDoesNotCompilePercentOfAttempted'] = 62.5
data['ToplasBuildRuns'] = 3
data['ToplasBuildRunsPercentOfAttempted'] = 37.5
data['ToplasBuildCompilesDoesNotRun'] = 0
data['ToplasBuildCompilesDoesNotRunPercentOfAttempted'] = 0.0
data['ToplasBuildOnlineRuns'] = 0
data['ToplasBuildOnlineRunsPercentOfAttempted'] = 0.0
data['ToplasBuildSuccess'] = 3
data['ToplasBuildSuccessPercentOfAttempted'] = 37.5
data['ToplasBuildSuccessPercentOfImplementation'] = 30.0
data['PldiBuildAttempted'] = 26
data['PldiBuildAttemptedPercentOfImplementation'] = 54.2
data['PldiBuildAttemptedPercentOfAvailableCode'] = 100.0
data['PldiBuildCompiles'] = 8
data['PldiBuildCompilesPercentOfAttempted'] = 30.8
data['PldiBuildDoesNotCompile'] = 18
data['PldiBuildDoesNotCompilePercentOfAttempted'] = 69.2
data['PldiBuildRuns'] = 3
data['PldiBuildRunsPercentOfAttempted'] = 11.5
data['PldiBuildCompilesDoesNotRun'] = 5
data['PldiBuildCompilesDoesNotRunPercentOfAttempted'] = 19.2
data['PldiBuildOnlineRuns'] = 1
data['PldiBuildOnlineRunsPercentOfAttempted'] = 3.8
data['PldiBuildSuccess'] = 4
data['PldiBuildSuccessPercentOfAttempted'] = 15.4
data['PldiBuildSuccessPercentOfImplementation'] = 8.3
data['SospBuildAttempted'] = 9
data['SospBuildAttemptedPercentOfImplementation'] = 34.6
data['SospBuildAttemptedPercentOfAvailableCode'] = 100.0
data['SospBuildCompiles'] = 2
data['SospBuildCompilesPercentOfAttempted'] = 22.2
data['SospBuildDoesNotCompile'] = 7
data['SospBuildDoesNotCompilePercentOfAttempted'] = 77.8
data['SospBuildRuns'] = 2
data['SospBuildRunsPercentOfAttempted'] = 22.2
data['SospBuildCompilesDoesNotRun'] = 0
data['SospBuildCompilesDoesNotRunPercentOfAttempted'] = 0.0
data['SospBuildOnlineRuns'] = 0
data['SospBuildOnlineRunsPercentOfAttempted'] = 0.0
data['SospBuildSuccess'] = 2
data['SospBuildSuccessPercentOfAttempted'] = 22.2
data['SospBuildSuccessPercentOfImplementation'] = 7.7
data['VldbBuildAttempted'] = 50
data['VldbBuildAttemptedPercentOfImplementation'] = 37.3
data['VldbBuildAttemptedPercentOfAvailableCode'] = 100.0
data['VldbBuildCompiles'] = 34
data['VldbBuildCompilesPercentOfAttempted'] = 68.0
data['VldbBuildDoesNotCompile'] = 16
data['VldbBuildDoesNotCompilePercentOfAttempted'] = 32.0
data['VldbBuildRuns'] = 27
data['VldbBuildRunsPercentOfAttempted'] = 54.0
data['VldbBuildCompilesDoesNotRun'] = 7
data['VldbBuildCompilesDoesNotRunPercentOfAttempted'] = 14.0
data['VldbBuildOnlineRuns'] = 0
data['VldbBuildOnlineRunsPercentOfAttempted'] = 0.0
data['VldbBuildSuccess'] = 27
data['VldbBuildSuccessPercentOfAttempted'] = 54.0
data['VldbBuildSuccessPercentOfImplementation'] = 20.1
data['TocsBuildAttempted'] = 6
data['TocsBuildAttemptedPercentOfImplementation'] = 46.2
data['TocsBuildAttemptedPercentOfAvailableCode'] = 100.0
data['TocsBuildCompiles'] = 1
data['TocsBuildCompilesPercentOfAttempted'] = 16.7
data['TocsBuildDoesNotCompile'] = 3
data['TocsBuildDoesNotCompilePercentOfAttempted'] = 50.0
data['TocsBuildRuns'] = 1
data['TocsBuildRunsPercentOfAttempted'] = 16.7
data['TocsBuildCompilesDoesNotRun'] = 0
data['TocsBuildCompilesDoesNotRunPercentOfAttempted'] = 0.0
data['TocsBuildOnlineRuns'] = 1
data['TocsBuildOnlineRunsPercentOfAttempted'] = 16.7
data['TocsBuildSuccess'] = 2
data['TocsBuildSuccessPercentOfAttempted'] = 33.3
data['TocsBuildSuccessPercentOfImplementation'] = 15.4
data['CcsBuildAttempted'] = 24
data['CcsBuildAttemptedPercentOfImplementation'] = 42.9
data['CcsBuildAttemptedPercentOfAvailableCode'] = 100.0
data['CcsBuildCompiles'] = 12
data['CcsBuildCompilesPercentOfAttempted'] = 50.0
data['CcsBuildDoesNotCompile'] = 12
data['CcsBuildDoesNotCompilePercentOfAttempted'] = 50.0
data['CcsBuildRuns'] = 8
data['CcsBuildRunsPercentOfAttempted'] = 33.3
data['CcsBuildCompilesDoesNotRun'] = 4
data['CcsBuildCompilesDoesNotRunPercentOfAttempted'] = 16.7
data['CcsBuildOnlineRuns'] = 0
data['CcsBuildOnlineRunsPercentOfAttempted'] = 0.0
data['CcsBuildSuccess'] = 8
data['CcsBuildSuccessPercentOfAttempted'] = 33.3
data['CcsBuildSuccessPercentOfImplementation'] = 14.3
data['JournalsBuildAttempted'] = 40
data['JournalsBuildAttemptedPercentOfImplementation'] = 47.1
data['JournalsBuildAttemptedPercentOfAvailableCode'] = 100.0
data['JournalsBuildCompiles'] = 17
data['JournalsBuildCompilesPercentOfAttempted'] = 42.5
data['JournalsBuildDoesNotCompile'] = 20
data['JournalsBuildDoesNotCompilePercentOfAttempted'] = 50.0
data['JournalsBuildRuns'] = 17
data['JournalsBuildRunsPercentOfAttempted'] = 42.5
data['JournalsBuildCompilesDoesNotRun'] = 0
data['JournalsBuildCompilesDoesNotRunPercentOfAttempted'] = 0.0
data['JournalsBuildOnlineRuns'] = 1
data['JournalsBuildOnlineRunsPercentOfAttempted'] = 2.5
data['JournalsBuildSuccess'] = 18
data['JournalsBuildSuccessPercentOfAttempted'] = 45.0
data['JournalsBuildSuccessPercentOfImplementation'] = 21.2
data['ConferencesBuildAttempted'] = 188
data['ConferencesBuildAttemptedPercentOfImplementation'] = 43.8
data['ConferencesBuildAttemptedPercentOfAvailableCode'] = 100.0
data['ConferencesBuildCompiles'] = 97
data['ConferencesBuildCompilesPercentOfAttempted'] = 51.6
data['ConferencesBuildDoesNotCompile'] = 88
data['ConferencesBuildDoesNotCompilePercentOfAttempted'] = 46.8
data['ConferencesBuildRuns'] = 76
data['ConferencesBuildRunsPercentOfAttempted'] = 40.4
data['ConferencesBuildCompilesDoesNotRun'] = 21
data['ConferencesBuildCompilesDoesNotRunPercentOfAttempted'] = 11.2
data['ConferencesBuildOnlineRuns'] = 4
data['ConferencesBuildOnlineRunsPercentOfAttempted'] = 2.1
data['ConferencesBuildSuccess'] = 80
data['ConferencesBuildSuccessPercentOfAttempted'] = 42.6
data['ConferencesBuildSuccessPercentOfImplementation'] = 18.6
data['CommercialNoneBuildAttempted'] = 177
data['CommercialNoneBuildAttemptedPercentOfImplementation'] = 50.7
data['CommercialNoneBuildAttemptedPercentOfAvailableCode'] = 99.4
data['CommercialNoneBuildCompiles'] = 89
data['CommercialNoneBuildCompilesPercentOfAttempted'] = 50.3
data['CommercialNoneBuildDoesNotCompile'] = 85
data['CommercialNoneBuildDoesNotCompilePercentOfAttempted'] = 48.0
data['CommercialNoneBuildRuns'] = 78
data['CommercialNoneBuildRunsPercentOfAttempted'] = 44.1
data['CommercialNoneBuildCompilesDoesNotRun'] = 11
data['CommercialNoneBuildCompilesDoesNotRunPercentOfAttempted'] = 6.2
data['CommercialNoneBuildOnlineRuns'] = 3
data['CommercialNoneBuildOnlineRunsPercentOfAttempted'] = 1.7
data['CommercialNoneBuildSuccess'] = 81
data['CommercialNoneBuildSuccessPercentOfAttempted'] = 45.8
data['CommercialNoneBuildSuccessPercentOfImplementation'] = 23.2
data['CommercialPartBuildAttempted'] = 43
data['CommercialPartBuildAttemptedPercentOfImplementation'] = 33.1
data['CommercialPartBuildAttemptedPercentOfAvailableCode'] = 102.4
data['CommercialPartBuildCompiles'] = 21
data['CommercialPartBuildCompilesPercentOfAttempted'] = 48.8
data['CommercialPartBuildDoesNotCompile'] = 19
data['CommercialPartBuildDoesNotCompilePercentOfAttempted'] = 44.2
data['CommercialPartBuildRuns'] = 12
data['CommercialPartBuildRunsPercentOfAttempted'] = 27.9
data['CommercialPartBuildCompilesDoesNotRun'] = 9
data['CommercialPartBuildCompilesDoesNotRunPercentOfAttempted'] = 20.9
data['CommercialPartBuildOnlineRuns'] = 2
data['CommercialPartBuildOnlineRunsPercentOfAttempted'] = 4.7
data['CommercialPartBuildSuccess'] = 14
data['CommercialPartBuildSuccessPercentOfAttempted'] = 32.6
data['CommercialPartBuildSuccessPercentOfImplementation'] = 10.8
data['CommercialFullBuildAttempted'] = 8
data['CommercialFullBuildAttemptedPercentOfImplementation'] = 22.9
data['CommercialFullBuildAttemptedPercentOfAvailableCode'] = 100.0
data['CommercialFullBuildCompiles'] = 4
data['CommercialFullBuildCompilesPercentOfAttempted'] = 50.0
data['CommercialFullBuildDoesNotCompile'] = 4
data['CommercialFullBuildDoesNotCompilePercentOfAttempted'] = 50.0
data['CommercialFullBuildRuns'] = 3
data['CommercialFullBuildRunsPercentOfAttempted'] = 37.5
data['CommercialFullBuildCompilesDoesNotRun'] = 1
data['CommercialFullBuildCompilesDoesNotRunPercentOfAttempted'] = 12.5
data['CommercialFullBuildOnlineRuns'] = 0
data['CommercialFullBuildOnlineRunsPercentOfAttempted'] = 0.0
data['CommercialFullBuildSuccess'] = 3
data['CommercialFullBuildSuccessPercentOfAttempted'] = 37.5
data['CommercialFullBuildSuccessPercentOfImplementation'] = 8.6
data['NoNSFBuildAttempted'] = 125
data['NoNSFBuildAttemptedPercentOfImplementation'] = 43.6
data['NoNSFBuildAttemptedPercentOfAvailableCode'] = 100.0
data['NoNSFBuildCompiles'] = 66
data['NoNSFBuildCompilesPercentOfAttempted'] = 52.8
data['NoNSFBuildDoesNotCompile'] = 58
data['NoNSFBuildDoesNotCompilePercentOfAttempted'] = 46.4
data['NoNSFBuildRuns'] = 54
data['NoNSFBuildRunsPercentOfAttempted'] = 43.2
data['NoNSFBuildCompilesDoesNotRun'] = 12
data['NoNSFBuildCompilesDoesNotRunPercentOfAttempted'] = 9.6
data['NoNSFBuildOnlineRuns'] = 1
data['NoNSFBuildOnlineRunsPercentOfAttempted'] = 0.8
data['NoNSFBuildSuccess'] = 55
data['NoNSFBuildSuccessPercentOfAttempted'] = 44.0
data['NoNSFBuildSuccessPercentOfImplementation'] = 19.2
data['NSFBuildAttempted'] = 103
data['NSFBuildAttemptedPercentOfImplementation'] = 45.4
data['NSFBuildAttemptedPercentOfAvailableCode'] = 100.0
data['NSFBuildCompiles'] = 48
data['NSFBuildCompilesPercentOfAttempted'] = 46.6
data['NSFBuildDoesNotCompile'] = 50
data['NSFBuildDoesNotCompilePercentOfAttempted'] = 48.5
data['NSFBuildRuns'] = 39
data['NSFBuildRunsPercentOfAttempted'] = 37.9
data['NSFBuildCompilesDoesNotRun'] = 9
data['NSFBuildCompilesDoesNotRunPercentOfAttempted'] = 8.7
data['NSFBuildOnlineRuns'] = 4
data['NSFBuildOnlineRunsPercentOfAttempted'] = 3.9
data['NSFBuildSuccess'] = 43
data['NSFBuildSuccessPercentOfAttempted'] = 41.7
data['NSFBuildSuccessPercentOfImplementation'] = 18.9
data['TotalBuildAttempted'] = 228
data['TotalBuildAttemptedPercentOfImplementation'] = 44.4
data['TotalBuildAttemptedPercentOfAvailableCode'] = 100.0
data['TotalBuildCompiles'] = 114
data['TotalBuildCompilesPercentOfAttempted'] = 50.0
data['TotalBuildDoesNotCompile'] = 108
data['TotalBuildDoesNotCompilePercentOfAttempted'] = 47.4
data['TotalBuildRuns'] = 93
data['TotalBuildRunsPercentOfAttempted'] = 40.8
data['TotalBuildCompilesDoesNotRun'] = 21
data['TotalBuildCompilesDoesNotRunPercentOfAttempted'] = 9.2
data['TotalBuildOnlineRuns'] = 5
data['TotalBuildOnlineRunsPercentOfAttempted'] = 2.2
data['TotalBuildSuccess'] = 98
data['TotalBuildSuccessPercentOfAttempted'] = 43.0
data['TotalBuildSuccessPercentOfImplementation'] = 19.1
data['TissecEmailNeeded'] = 3
data['TissecEmailSent'] = 3
data['TissecEmailSentPercentOfNeeded'] = 100.0
data['TissecEmailNeededNotSent'] = 0
data['TissecEmailNeededNotSentPercentOfNeeded'] = 0.0
data['TissecEmailGotReply'] = 3
data['TissecEmailGotReplyPercentOfSent'] = 100.0
data['TissecEmailWithoutReply'] = 0
data['TissecEmailWithoutReplyPercentOfSent'] = 0.0
data['TissecEmailGotReplyYes'] = 1
data['TissecEmailGotReplyYesPercentOfSent'] = 33.3
data['TissecEmailGotReplyYesPercentOfReply'] = 33.3
data['TissecEmailGotReplyNo'] = 2
data['TissecEmailGotReplyNoPercentOfSent'] = 66.7
data['TissecEmailGotReplyNoPercentOfReply'] = 66.7
data['TacoEmailNeeded'] = 29
data['TacoEmailSent'] = 27
data['TacoEmailSentPercentOfNeeded'] = 93.1
data['TacoEmailNeededNotSent'] = 2
data['TacoEmailNeededNotSentPercentOfNeeded'] = 6.9
data['TacoEmailGotReply'] = 21
data['TacoEmailGotReplyPercentOfSent'] = 77.8
data['TacoEmailWithoutReply'] = 6
data['TacoEmailWithoutReplyPercentOfSent'] = 22.2
data['TacoEmailGotReplyYes'] = 4
data['TacoEmailGotReplyYesPercentOfSent'] = 14.8
data['TacoEmailGotReplyYesPercentOfReply'] = 19.0
data['TacoEmailGotReplyNo'] = 17
data['TacoEmailGotReplyNoPercentOfSent'] = 63.0
data['TacoEmailGotReplyNoPercentOfReply'] = 81.0
data['TodsEmailNeeded'] = 11
data['TodsEmailSent'] = 11
data['TodsEmailSentPercentOfNeeded'] = 100.0
data['TodsEmailNeededNotSent'] = 0
data['TodsEmailNeededNotSentPercentOfNeeded'] = 0.0
data['TodsEmailGotReply'] = 6
data['TodsEmailGotReplyPercentOfSent'] = 54.5
data['TodsEmailWithoutReply'] = 5
data['TodsEmailWithoutReplyPercentOfSent'] = 45.5
data['TodsEmailGotReplyYes'] = 2
data['TodsEmailGotReplyYesPercentOfSent'] = 18.2
data['TodsEmailGotReplyYesPercentOfReply'] = 33.3
data['TodsEmailGotReplyNo'] = 4
data['TodsEmailGotReplyNoPercentOfSent'] = 36.4
data['TodsEmailGotReplyNoPercentOfReply'] = 66.7
data['OsdiEmailNeeded'] = 16
data['OsdiEmailSent'] = 9
data['OsdiEmailSentPercentOfNeeded'] = 56.3
data['OsdiEmailNeededNotSent'] = 7
data['OsdiEmailNeededNotSentPercentOfNeeded'] = 43.8
data['OsdiEmailGotReply'] = 9
data['OsdiEmailGotReplyPercentOfSent'] = 100.0
data['OsdiEmailWithoutReply'] = 0
data['OsdiEmailWithoutReplyPercentOfSent'] = 0.0
data['OsdiEmailGotReplyYes'] = 2
data['OsdiEmailGotReplyYesPercentOfSent'] = 22.2
data['OsdiEmailGotReplyYesPercentOfReply'] = 22.2
data['OsdiEmailGotReplyNo'] = 7
data['OsdiEmailGotReplyNoPercentOfSent'] = 77.8
data['OsdiEmailGotReplyNoPercentOfReply'] = 77.8
data['SigmodEmailNeeded'] = 36
data['SigmodEmailSent'] = 16
data['SigmodEmailSentPercentOfNeeded'] = 44.4
data['SigmodEmailNeededNotSent'] = 20
data['SigmodEmailNeededNotSentPercentOfNeeded'] = 55.6
data['SigmodEmailGotReply'] = 14
data['SigmodEmailGotReplyPercentOfSent'] = 87.5
data['SigmodEmailWithoutReply'] = 2
data['SigmodEmailWithoutReplyPercentOfSent'] = 12.5
data['SigmodEmailGotReplyYes'] = 8
data['SigmodEmailGotReplyYesPercentOfSent'] = 50.0
data['SigmodEmailGotReplyYesPercentOfReply'] = 57.1
data['SigmodEmailGotReplyNo'] = 7
data['SigmodEmailGotReplyNoPercentOfSent'] = 43.8
data['SigmodEmailGotReplyNoPercentOfReply'] = 50.0
data['AsplosEmailNeeded'] = 27
data['AsplosEmailSent'] = 20
data['AsplosEmailSentPercentOfNeeded'] = 74.1
data['AsplosEmailNeededNotSent'] = 7
data['AsplosEmailNeededNotSentPercentOfNeeded'] = 25.9
data['AsplosEmailGotReply'] = 19
data['AsplosEmailGotReplyPercentOfSent'] = 95.0
data['AsplosEmailWithoutReply'] = 1
data['AsplosEmailWithoutReplyPercentOfSent'] = 5.0
data['AsplosEmailGotReplyYes'] = 4
data['AsplosEmailGotReplyYesPercentOfSent'] = 20.0
data['AsplosEmailGotReplyYesPercentOfReply'] = 21.1
data['AsplosEmailGotReplyNo'] = 15
data['AsplosEmailGotReplyNoPercentOfSent'] = 75.0
data['AsplosEmailGotReplyNoPercentOfReply'] = 78.9
data['OopslaEmailNeeded'] = 30
data['OopslaEmailSent'] = 25
data['OopslaEmailSentPercentOfNeeded'] = 83.3
data['OopslaEmailNeededNotSent'] = 5
data['OopslaEmailNeededNotSentPercentOfNeeded'] = 16.7
data['OopslaEmailGotReply'] = 24
data['OopslaEmailGotReplyPercentOfSent'] = 96.0
data['OopslaEmailWithoutReply'] = 1
data['OopslaEmailWithoutReplyPercentOfSent'] = 4.0
data['OopslaEmailGotReplyYes'] = 9
data['OopslaEmailGotReplyYesPercentOfSent'] = 36.0
data['OopslaEmailGotReplyYesPercentOfReply'] = 37.5
data['OopslaEmailGotReplyNo'] = 15
data['OopslaEmailGotReplyNoPercentOfSent'] = 60.0
data['OopslaEmailGotReplyNoPercentOfReply'] = 62.5
data['ToplasEmailNeeded'] = 3
data['ToplasEmailSent'] = 1
data['ToplasEmailSentPercentOfNeeded'] = 33.3
data['ToplasEmailNeededNotSent'] = 2
data['ToplasEmailNeededNotSentPercentOfNeeded'] = 66.7
data['ToplasEmailGotReply'] = 1
data['ToplasEmailGotReplyPercentOfSent'] = 100.0
data['ToplasEmailWithoutReply'] = 0
data['ToplasEmailWithoutReplyPercentOfSent'] = 0.0
data['ToplasEmailGotReplyYes'] = 1
data['ToplasEmailGotReplyYesPercentOfSent'] = 100.0
data['ToplasEmailGotReplyYesPercentOfReply'] = 100.0
data['ToplasEmailGotReplyNo'] = 0
data['ToplasEmailGotReplyNoPercentOfSent'] = 0.0
data['ToplasEmailGotReplyNoPercentOfReply'] = 0.0
data['PldiEmailNeeded'] = 32
data['PldiEmailSent'] = 25
data['PldiEmailSentPercentOfNeeded'] = 78.1
data['PldiEmailNeededNotSent'] = 7
data['PldiEmailNeededNotSentPercentOfNeeded'] = 21.9
data['PldiEmailGotReply'] = 24
data['PldiEmailGotReplyPercentOfSent'] = 96.0
data['PldiEmailWithoutReply'] = 1
data['PldiEmailWithoutReplyPercentOfSent'] = 4.0
data['PldiEmailGotReplyYes'] = 10
data['PldiEmailGotReplyYesPercentOfSent'] = 40.0
data['PldiEmailGotReplyYesPercentOfReply'] = 41.7
data['PldiEmailGotReplyNo'] = 14
data['PldiEmailGotReplyNoPercentOfSent'] = 56.0
data['PldiEmailGotReplyNoPercentOfReply'] = 58.3
data['SospEmailNeeded'] = 20
data['SospEmailSent'] = 13
data['SospEmailSentPercentOfNeeded'] = 65.0
data['SospEmailNeededNotSent'] = 7
data['SospEmailNeededNotSentPercentOfNeeded'] = 35.0
data['SospEmailGotReply'] = 11
data['SospEmailGotReplyPercentOfSent'] = 84.6
data['SospEmailWithoutReply'] = 2
data['SospEmailWithoutReplyPercentOfSent'] = 15.4
data['SospEmailGotReplyYes'] = 3
data['SospEmailGotReplyYesPercentOfSent'] = 23.1
data['SospEmailGotReplyYesPercentOfReply'] = 27.3
data['SospEmailGotReplyNo'] = 8
data['SospEmailGotReplyNoPercentOfSent'] = 61.5
data['SospEmailGotReplyNoPercentOfReply'] = 72.7
data['VldbEmailNeeded'] = 104
data['VldbEmailSent'] = 74
data['VldbEmailSentPercentOfNeeded'] = 71.2
data['VldbEmailNeededNotSent'] = 30
data['VldbEmailNeededNotSentPercentOfNeeded'] = 28.8
data['VldbEmailGotReply'] = 63
data['VldbEmailGotReplyPercentOfSent'] = 85.1
data['VldbEmailWithoutReply'] = 11
data['VldbEmailWithoutReplyPercentOfSent'] = 14.9
data['VldbEmailGotReplyYes'] = 20
data['VldbEmailGotReplyYesPercentOfSent'] = 27.0
data['VldbEmailGotReplyYesPercentOfReply'] = 31.7
data['VldbEmailGotReplyNo'] = 43
data['VldbEmailGotReplyNoPercentOfSent'] = 58.1
data['VldbEmailGotReplyNoPercentOfReply'] = 68.3
data['TocsEmailNeeded'] = 8
data['TocsEmailSent'] = 8
data['TocsEmailSentPercentOfNeeded'] = 100.0
data['TocsEmailNeededNotSent'] = 0
data['TocsEmailNeededNotSentPercentOfNeeded'] = 0.0
data['TocsEmailGotReply'] = 6
data['TocsEmailGotReplyPercentOfSent'] = 75.0
data['TocsEmailWithoutReply'] = 2
data['TocsEmailWithoutReplyPercentOfSent'] = 25.0
data['TocsEmailGotReplyYes'] = 1
data['TocsEmailGotReplyYesPercentOfSent'] = 12.5
data['TocsEmailGotReplyYesPercentOfReply'] = 16.7
data['TocsEmailGotReplyNo'] = 5
data['TocsEmailGotReplyNoPercentOfSent'] = 62.5
data['TocsEmailGotReplyNoPercentOfReply'] = 83.3
data['CcsEmailNeeded'] = 46
data['CcsEmailSent'] = 28
data['CcsEmailSentPercentOfNeeded'] = 60.9
data['CcsEmailNeededNotSent'] = 18
data['CcsEmailNeededNotSentPercentOfNeeded'] = 39.1
data['CcsEmailGotReply'] = 26
data['CcsEmailGotReplyPercentOfSent'] = 92.9
data['CcsEmailWithoutReply'] = 2
data['CcsEmailWithoutReplyPercentOfSent'] = 7.1
data['CcsEmailGotReplyYes'] = 14
data['CcsEmailGotReplyYesPercentOfSent'] = 50.0
data['CcsEmailGotReplyYesPercentOfReply'] = 53.8
data['CcsEmailGotReplyNo'] = 12
data['CcsEmailGotReplyNoPercentOfSent'] = 42.9
data['CcsEmailGotReplyNoPercentOfReply'] = 46.2
data['JournalsEmailNeeded'] = 54
data['JournalsEmailSent'] = 50
data['JournalsEmailSentPercentOfNeeded'] = 92.6
data['JournalsEmailNeededNotSent'] = 4
data['JournalsEmailNeededNotSentPercentOfNeeded'] = 7.4
data['JournalsEmailGotReply'] = 37
data['JournalsEmailGotReplyPercentOfSent'] = 74.0
data['JournalsEmailWithoutReply'] = 13
data['JournalsEmailWithoutReplyPercentOfSent'] = 26.0
data['JournalsEmailGotReplyYes'] = 9
data['JournalsEmailGotReplyYesPercentOfSent'] = 18.0
data['JournalsEmailGotReplyYesPercentOfReply'] = 24.3
data['JournalsEmailGotReplyNo'] = 28
data['JournalsEmailGotReplyNoPercentOfSent'] = 56.0
data['JournalsEmailGotReplyNoPercentOfReply'] = 75.7
data['ConferencesEmailNeeded'] = 311
data['ConferencesEmailSent'] = 210
data['ConferencesEmailSentPercentOfNeeded'] = 67.5
data['ConferencesEmailNeededNotSent'] = 101
data['ConferencesEmailNeededNotSentPercentOfNeeded'] = 32.5
data['ConferencesEmailGotReply'] = 190
data['ConferencesEmailGotReplyPercentOfSent'] = 90.5
data['ConferencesEmailWithoutReply'] = 20
data['ConferencesEmailWithoutReplyPercentOfSent'] = 9.5
data['ConferencesEmailGotReplyYes'] = 70
data['ConferencesEmailGotReplyYesPercentOfSent'] = 33.3
data['ConferencesEmailGotReplyYesPercentOfReply'] = 36.8
data['ConferencesEmailGotReplyNo'] = 121
data['ConferencesEmailGotReplyNoPercentOfSent'] = 57.6
data['ConferencesEmailGotReplyNoPercentOfReply'] = 63.7
data['CommercialNoneEmailNeeded'] = 231
data['CommercialNoneEmailSent'] = 168
data['CommercialNoneEmailSentPercentOfNeeded'] = 72.7
data['CommercialNoneEmailNeededNotSent'] = 63
data['CommercialNoneEmailNeededNotSentPercentOfNeeded'] = 27.3
data['CommercialNoneEmailGotReply'] = 146
data['CommercialNoneEmailGotReplyPercentOfSent'] = 86.9
data['CommercialNoneEmailWithoutReply'] = 22
data['CommercialNoneEmailWithoutReplyPercentOfSent'] = 13.1
data['CommercialNoneEmailGotReplyYes'] = 60
data['CommercialNoneEmailGotReplyYesPercentOfSent'] = 35.7
data['CommercialNoneEmailGotReplyYesPercentOfReply'] = 41.1
data['CommercialNoneEmailGotReplyNo'] = 87
data['CommercialNoneEmailGotReplyNoPercentOfSent'] = 51.8
data['CommercialNoneEmailGotReplyNoPercentOfReply'] = 59.6
data['CommercialPartEmailNeeded'] = 105
data['CommercialPartEmailSent'] = 69
data['CommercialPartEmailSentPercentOfNeeded'] = 65.7
data['CommercialPartEmailNeededNotSent'] = 36
data['CommercialPartEmailNeededNotSentPercentOfNeeded'] = 34.3
data['CommercialPartEmailGotReply'] = 61
data['CommercialPartEmailGotReplyPercentOfSent'] = 88.4
data['CommercialPartEmailWithoutReply'] = 8
data['CommercialPartEmailWithoutReplyPercentOfSent'] = 11.6
data['CommercialPartEmailGotReplyYes'] = 17
data['CommercialPartEmailGotReplyYesPercentOfSent'] = 24.6
data['CommercialPartEmailGotReplyYesPercentOfReply'] = 27.9
data['CommercialPartEmailGotReplyNo'] = 44
data['CommercialPartEmailGotReplyNoPercentOfSent'] = 63.8
data['CommercialPartEmailGotReplyNoPercentOfReply'] = 72.1
data['CommercialFullEmailNeeded'] = 29
data['CommercialFullEmailSent'] = 23
data['CommercialFullEmailSentPercentOfNeeded'] = 79.3
data['CommercialFullEmailNeededNotSent'] = 6
data['CommercialFullEmailNeededNotSentPercentOfNeeded'] = 20.7
data['CommercialFullEmailGotReply'] = 20
data['CommercialFullEmailGotReplyPercentOfSent'] = 87.0
data['CommercialFullEmailWithoutReply'] = 3
data['CommercialFullEmailWithoutReplyPercentOfSent'] = 13.0
data['CommercialFullEmailGotReplyYes'] = 2
data['CommercialFullEmailGotReplyYesPercentOfSent'] = 8.7
data['CommercialFullEmailGotReplyYesPercentOfReply'] = 10.0
data['CommercialFullEmailGotReplyNo'] = 18
data['CommercialFullEmailGotReplyNoPercentOfSent'] = 78.3
data['CommercialFullEmailGotReplyNoPercentOfReply'] = 90.0
data['NoNSFEmailNeeded'] = 204
data['NoNSFEmailSent'] = 153
data['NoNSFEmailSentPercentOfNeeded'] = 75.0
data['NoNSFEmailNeededNotSent'] = 51
data['NoNSFEmailNeededNotSentPercentOfNeeded'] = 25.0
data['NoNSFEmailGotReply'] = 134
data['NoNSFEmailGotReplyPercentOfSent'] = 87.6
data['NoNSFEmailWithoutReply'] = 19
data['NoNSFEmailWithoutReplyPercentOfSent'] = 12.4
data['NoNSFEmailGotReplyYes'] = 42
data['NoNSFEmailGotReplyYesPercentOfSent'] = 27.5
data['NoNSFEmailGotReplyYesPercentOfReply'] = 31.3
data['NoNSFEmailGotReplyNo'] = 92
data['NoNSFEmailGotReplyNoPercentOfSent'] = 60.1
data['NoNSFEmailGotReplyNoPercentOfReply'] = 68.7
data['NSFEmailNeeded'] = 161
data['NSFEmailSent'] = 107
data['NSFEmailSentPercentOfNeeded'] = 66.5
data['NSFEmailNeededNotSent'] = 54
data['NSFEmailNeededNotSentPercentOfNeeded'] = 33.5
data['NSFEmailGotReply'] = 93
data['NSFEmailGotReplyPercentOfSent'] = 86.9
data['NSFEmailWithoutReply'] = 14
data['NSFEmailWithoutReplyPercentOfSent'] = 13.1
data['NSFEmailGotReplyYes'] = 37
data['NSFEmailGotReplyYesPercentOfSent'] = 34.6
data['NSFEmailGotReplyYesPercentOfReply'] = 39.8
data['NSFEmailGotReplyNo'] = 57
data['NSFEmailGotReplyNoPercentOfSent'] = 53.3
data['NSFEmailGotReplyNoPercentOfReply'] = 61.3
data['TotalEmailNeeded'] = 365
data['TotalEmailSent'] = 260
data['TotalEmailSentPercentOfNeeded'] = 71.2
data['TotalEmailNeededNotSent'] = 105
data['TotalEmailNeededNotSentPercentOfNeeded'] = 28.8
data['TotalEmailGotReply'] = 227
data['TotalEmailGotReplyPercentOfSent'] = 87.3
data['TotalEmailWithoutReply'] = 33
data['TotalEmailWithoutReplyPercentOfSent'] = 12.7
data['TotalEmailGotReplyYes'] = 79
data['TotalEmailGotReplyYesPercentOfSent'] = 30.4
data['TotalEmailGotReplyYesPercentOfReply'] = 34.8
data['TotalEmailGotReplyNo'] = 149
data['TotalEmailGotReplyNoPercentOfSent'] = 57.3
data['TotalEmailGotReplyNoPercentOfReply'] = 65.6
##################################################################
def title(t,w):
   if titleOn:
      return t + str(data[what + w])
   else:
      return str(data[what + w])

def titleSum(t,w1,w2):
   if titleOn:
      return t + str(data[what + w1] + data[what + w2])
   else:
      return str(data[what + w1] + data[what + w2])

def fraction(w):
   return data[what + w]/data[what + 'Total']

def fractionSum(w1,w2):
   return (data[what + w1]+data[what + w2])/data[what + 'Total']

##################################################################

fig = plt.figure()
ax = fig.add_subplot(1, 1, 1, xticks=[], yticks=[], title="")
sankey = Sankey(ax=ax, 
                scale=SCALE, 
                offset=0.15, 
#                gap=0.2,
                head_angle=90,
                margin=0.0,
                format='%.0f', 
                unit=None)

########################################################################
# diagram 0
########################################################################
sankey.add(
   flows=[
      fraction('Total'),
         -fraction('WithNoImplementation'),
         -fraction('WithImplementation'),
    ],
    labels=[
       title('', 'Total'), 
       title('Theoretical\n','WithNoImplementation'), 
        '', 
    ],
    orientations=[
         0, 1, 0
    ],
    pathlengths = [0.0, 0.2, 0.0],
    linewidth=0.0,
    trunklength=1.0
)
########################################################################
# diagram 1
########################################################################
sankey.add(
   flows=[
       fraction('WithImplementation'),
          -fraction('WithArticleCode'),
          -fraction('WithGoogleCode'),
          -fraction('WithEmailCode'),
          -fractionSum('EmailWithoutReply','EmailGotReplyNo'),
          -fraction('EmailNeededNotSent'),
    ],
    labels=[
       title('', 'WithImplementation'), 
       title('Article\n', 'WithArticleCode'), 
       title('Web\n', 'WithGoogleCode'), 
       title('Email\n', 'WithEmailCode'), 
       '',
       title('Email\nNot Sent\n', 'EmailNeededNotSent'), 
    ],
    orientations=[
         0, 1, 1, 1, 0, -1
    ],
   prior=0, 
   connect=(2, 0),
   trunklength = 0.5,
   linewidth=0.0,
   pathlengths = [0.1, 0.09, 0.21, 0.29, 0.1, 0.15]
)
########################################################################
# diagram 2
########################################################################
sankey.add(
   flows=[
          fraction('WithArticleCode'),
          fraction('WithGoogleCode'),
          fraction('WithEmailCode'),
          -fraction('WithAvailableCode'),
    ],
    labels=[
       '',
       '',
       '',
       ''
    ],
    orientations=[
         -1, -1, -1, 0
    ],
   prior=1, 
   connect=(1, 0),
   trunklength = 0.6,
   linewidth=0.0,
   pathlengths = [0.5, 0.5, 0.5, 0.0]
)
 
########################################################################
# diagram 3
########################################################################
sankey.add(
   flows=[
       fraction('WithAvailableCode'),
           -fraction('BuildDoesNotCompile'),
           -fraction('BuildCompilesDoesNotRun'),
           -fraction('BuildRuns'),
           -fraction('BuildOnlineRuns'),
    ],
    labels=[
       title('', 'WithAvailableCode'),
       title('Build\nfails\n', 'BuildDoesNotCompile'), 
       title('Run\nfails\n', 'BuildCompilesDoesNotRun'), 
       title('Builds\n& runs\n', 'BuildRuns'), 
       title('Online\nruns\n', 'BuildOnlineRuns'),
    ],
    orientations=[
        0, -1, -1, 1, 1
    ],
   prior=2, 
   connect=(3, 0),
   trunklength = 0.6,
   linewidth=0.0,
   pathlengths = [0.0, 0.4, 0.2, 0.1, 0.1]
)
########################################################################
# diagram 4
########################################################################
sankey.add(
   flows=[
       fractionSum('EmailWithoutReply', 'EmailGotReplyNo'),
          -fraction('EmailGotReplyNo'),
          -fraction('EmailWithoutReply'),
    ],
    labels=[
       titleSum('','EmailWithoutReply', 'EmailGotReplyNo'), 
       title('Email\nreply=no\n','EmailGotReplyNo'), 
       title('No email\nreply\n', 'EmailWithoutReply'), 
    ],
    orientations=[
         0, -1, -1
    ],
   prior=1, 
   connect=(4, 0),
   trunklength = 0.7,
   linewidth=0.0,
   pathlengths = [0.0, 0.3, 0.2]
)

########################################################################
diagrams = sankey.finish()

diagrams[0].patch.set_facecolor('#80b060')
diagrams[1].patch.set_facecolor('#90c070')
diagrams[2].patch.set_facecolor('#90c070')
diagrams[3].patch.set_facecolor('#a0d080') 
diagrams[4].patch.set_facecolor('#a0d080') 

diagrams[1].texts[4].set_color('r')
diagrams[1].texts[5].set_color('b')
diagrams[3].texts[1].set_color('r')
diagrams[3].texts[2].set_color('r')
diagrams[4].texts[1].set_color('r')
diagrams[4].texts[2].set_color('r')

fig.patch.set_visible(False)
ax.axis('off')
fig.tight_layout()
plt.savefig(filename=OutputFile,frameon=False) 
#plt.savefig(filename='sankey-overview.png',frameon=False) 
#plt.show()
